--******************************************************************************
--*                                                                            *
--*  delMinorVersions v1.1            for Max 4.2                              *
--*  by Ofer Zelichover (c) 01/2002                                            *
--*  www.tdp.nu/ofer   ;   ofer_z@hotmail.com                                  *
--*                                                                            *
--******************************************************************************
--*  You may use this script freely as you see fit.                            *
--*  You may use parts or the script as a whole in your own scripts.           *
--*  (it would be nice if you give me a credit if you do so ;))                *
--******************************************************************************
--*  This script comes with no waranty!                                        *
--*  Although I tried this script and couldn't find any problems with it, I can*
--*  in no way be held responsible for any kind of loss or damage, whether     *
--*  direct or indirect, due to the use of this script.                        *
--*                                                                            *
--*  ********************************************************************      *
--*  *** IF YOU DON'T LIKE THE ABOVE STATEMENT, DON'T USE THIS SCRIPT ***      *
--*  ********************************************************************      *
--*                                                                            *
--*  **** This script was written for max 4.2 and wasn't tested on other ****  *
--*  **** versions of max.                                               ****  *
--*                                                                            *
--*  If you find any bugs in this script, please let me know.                  *
--******************************************************************************
--* Description                                                                *
--* -------------                                                              *
--*  This utility allows to easily delete minor versions that were created     *
--*  using the safeSave Minor Increment utility.                               *
--*                                                                            *
--******************************************************************************
--* History, Status and Known issues                                           *
--* ----------------------------------                                         *
--* Created: 31/01/2002 - Ver 1.0                                              *
--* Updated: 02/02/2002 - Ver 1.1                                              *
--*    - Added option to archive files.                                        *
--*    - Fixed minor bugs.                                                     *
--*                                                                            *
--* Known issues:                                                              *
--*                                                                            *
--* Need to add:                                                               *
--*                                                                            *
--******************************************************************************
--* Isntallation:                                                              *
--* --------------                                                             *
--*    put:                                                                    *
--*         delMinorVersions-v1.ms - anywhere                                  *
--*                                                                            *
--*  Just run the script.                                                      *
--*                                                                            *
--******************************************************************************

try(destroyDialog delMinorVersions)catch()

rollout delMinorVersions "Delete Minor Versions 1.1"
(
-- GUI
--------
	group "Base Flie: " (
		edittext baseFile ""  width:350 across:2
		button baseFileBrowse "Browse..." align:#right height:16
	)
	group "Minor Versions: " (
		multilistbox deleteFilesList "Delete Files: " width:200 height:20 align:#left across:2
		multilistbox keepFilesList "Keep Files: " width:200 height:20 align:#right
		button moveDeleteToKeep ">>" width:20 height:20 offset:[0,-170]
		button moveKeepToDelete "<<" width:20 height:20 offset:[0,10]
		button delFilesAll "All" height:16 width:50 pos:[15,365]
		button delFilesNone "None" height:16 width:50 pos:[88,365]
		button delFilesInvert "Invert" height:16 width:50 pos:[160,365]
		button keepFilesAll "All" height:16 width:50 pos:[240,365]
		button keepFilesNone "None" height:16 width:50 pos:[312,365]
		button keepFilesInvert "Invert" height:16 width:50 pos:[385,365]
	)

	group "Actions: " (
		checkbox actDelete "Delete Files" checked:true
		checkbox actArchive "Archive Files Into:" checked:false across:3
		edittext zipFilename "" offset:[-37,0] width:250 enabled:false
		button zipFileBrowse "Browse..." align:#right height:16 enabled:false

	)

	button bnGo "Execute" pos:[5,460] width:200
	button bnClose "Close" width:75 pos:[370,460]

-- Functions
-------------
-- Inverts the selection in a listbox
	fn selectionInvert sel = (
		#{1..sel.count}-sel
	)

	fn moveToKeep sel = (
		local newKeepList = keepFilesList.items
		local newDelList = deleteFilesList.items
		for i = newDelList.count to 1 by -1 where sel[i] do (
			append newKeepList newDelList[i]
			deleteItem newDelList i
		)
		keepFilesList.items = sort newKeepList
		deleteFilesList.items = sort newDelList
	)
	
	fn moveToDelete sel = (
		local newKeepList = keepFilesList.items
		local newDelList = deleteFilesList.items
		for i = newKeepList.count to 1 by -1 where sel[i] do (
			append newDelList newKeepList[i]
			deleteItem newKeepList i
		)
		keepFilesList.items = sort newKeepList
		deleteFilesList.items = sort newDelList
	)

	fn toLowerCase str = (
		local lowerCase = "abcdefghijklmnopqrstuvwxyz"
		local upperCase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		local outStr = ""
		for i = 1 to str.count do (
			local n = findString upperCase str[i]
			if n != undefined then outStr += lowerCase[n]
			else outStr += str[i]
		)
		outStr
	)
	
	fn isMinorVersion f = (
		local fName = getFilenameFile f
		local digits = "0123456789"
		local letters = "abcdefghijklmnopqrstuvwxyz"
		local i = findString letters (toLowerCase fName[fName.count])
		(i != undefined) AND (findString digits fName[fName.count-1] != undefined)
	)

	fn isMajorVersion f = (
		local fName = getFilenameFile f
		local digits = "0123456789"
		findString digits fName[fName.count] != undefined
	)
	
	fn getStrippedFilename f = (
		try (
			local fName = getFilenameFile f
			local fExt = getFilenameType f
			local fPath = getFilenamePath f
			local digits = "0123456789"
			local letters = "abcdefghijklmnopqrstuvwxyz"
			try (
				if isMinorVersion fName then 
					fName = subString fName 1 (fName.count-1)
				if isMajorVersion fName then (
					while (findString digits fName[fName.count] != undefined) do
						fName = subString fName 1 (fName.count-1)
				)
			)catch()
			(fPath + fName + fExt)
		)catch("")
	)

	fn getFileNames baseName = (
		if baseName == "" then return #()
		local basePath = getFilenamePath baseName
		local baseFilename = getFilenameFile baseName
		local baseExt = getFilenameType baseName
		local files = getFiles (basePath + getStrippedFilename baseFilename + "*" + baseExt)
		for i = files.count to 1 by -1 do (
			if isMinorVersion files[i] then
				files[i] = getFilenameFile files[i]
			else
				deleteItem files i
		)
		sort files
	)

	fn updateLists = (
		local files = getFileNames baseFile.text
		local keepList = #()
		local f = 0
		for i in keepFilesList.items where ((f=findItem files i) > 0) do (
			append keepList i
			deleteItem files f
		)
		deleteFilesList.items = sort files
		keepFilesList.items = sort keepList
	)

	fn getAutoZipName = (
		local date = (filterString localTime " ")[1]
		for i = 1 to date.count where date[i]=="/" do date[i]="-"
		local baseName = baseFile.text
		local fName = getFilenameFile baseName
		
		fName + "_" + date + ".zip"
	)

	fn getArchived resultFile = (
		local f = openFile resultFile mode:"r"
		returnVal = #()
		if f == undefined then return false
		try (
			local archived = #()
			while NOT (EOF f) do (
				local l = readLine f
				if (findString l "updating:")!=undefined OR (findString l "adding:")!=undefined then (
					append archived (getFilenameFile l)
				)
			)
		)catch()
		close f
		for i in deleteFilesList.items do
			append returnVal (findItem archived i > 0)
		returnVal
	)

	fn doArchive &resultFile = (
		local fName = sysInfo.tempDir + getFilenameFile zipFilename.text + ".lst"
		local f = createFile fName
		if f == undefined then return false
		local fExt = getFilenameType baseFile.text
		local fPath = getFilenamePath baseFile.text
		for i in deleteFilesList.items do (
			format "%\n" (fPath + i + fExt) to:f
		)
		flush f
		close f
		local zipUtil = getDir #maxRoot + "maxzip.exe"
		resultFile = sysInfo.tempDir + getFilenameFile zipFilename.text + ".rsl"
		local archive = dosCommand (zipUtil + " " + zipFilename.text + " @" + fName + ">" + resultFile)
		gc()
		deleteFile fName
		if archive==0 then return true else return flase
	)

	fn doDelete = (
		local resultFile = ""
		if actArchive.checked then 
			if NOT doArchive &resultFile then (
				messagebox "Archive failed." title:"Delete Minor Versions"
				return undefined
			)
		local fExt = getFilenameType baseFile.text
		local fPath = getFilenamePath baseFile.text
		local i = 0
		if fExt == "" then fExt = ".max"
		local archived = getArchived resultFile
		for f = 1 to deleteFilesList.items.count where (actDelete.checked AND (NOT actArchive.checked OR archived[f])) do (
			format "Deleting file: %\n" (fPath + deleteFilesList.items[f] + fExt)
			if deleteFile (fPath + deleteFilesList.items[f] + fExt) then
				i += 1
		)
		deleteFile resultFile
		messageBox ("Deleted "+i as string+" files.") title:"Finished Deleting."
	)

-- Event Handlers
------------------
	on baseFile entered text do (
		local f = getStrippedFilename text
		baseFile.text = f
		updateLists()
	)

	on baseFileBrowse pressed do(
		local f = getOpenFileName caption:"Select Base File" types:"MAX Files|*.max|All Files|*.*"
		if f != undefined then (
			baseFile.text = getStrippedFilename f
			updateLists()
		)
	)

	on deleteFilesList doubleClicked sel do (
		local fExt = getFilenameType (baseFile.text)
		local fPath = getFilenamePath (baseFile.text)
		if fExt == "" then fExt = ".max"
		local f = fPath + getStrippedFilename (deleteFilesList.items[sel]) + fExt
		baseFile.text = f
		updateLists()
	)

	on keepFilesList doubleClicked sel do (
		local fExt = getFilenameType (baseFile.text)
		local fPath = getFilenamePath (baseFile.text)
		if fExt == "" then fExt = ".max"
		local f = fPath + getStrippedFilename (keepFilesList.items[sel]) + fExt
		baseFile.text = f
		updateLists()
	)

	on moveDeleteToKeep pressed do (
		moveToKeep deleteFilesList.selection
		deleteFilesList.selection = #{}
	)
	on moveKeepToDelete pressed do (
		moveToDelete keepFilesList.selection
		keepFilesList.selection = #{}
	)
	on delFilesAll pressed do (deleteFilesList.selection=#{1..deleteFilesList.items.count})
	on delFilesNone pressed do (deleteFilesList.selection=#{})
	on delFilesInvert pressed do (deleteFilesList.selection=selectionInvert deleteFilesList.selection)
	on keepFilesAll pressed do (keepFilesList.selection=#{1..keepFilesList.items.count})
	on keepFilesNone pressed do (keepFilesList.selection=#{})
	on keepFilesInvert pressed do (keepFilesList.selection=selectionInvert keepFilesList.selection)
	
	on actDelete changed state do (
		bnGo.enabled = actArchive.checked OR actDelete.checked
	)

	on actArchive changed state do (
		zipFilename.enabled = state
		zipFileBrowse.enabled = state
		if zipFilename.text == "" then (
			local basePath = getFilenamePath baseFile.text
			zipFilename.text = basePath + getAutoZipName()
		)
		bnGo.enabled = actArchive.checked OR actDelete.checked
	)
		
	on zipFileBrowse pressed do (
		local autoFile = if zipFilename.text == "" then getAutoZipName() else zipFilename.text
		local f = getSaveFileName caption:"Select Archive File" types:"ZIP Files|*.zip|All Files|*.*" \
								filename:autoFile
		if f != undefined then 
			zipFilename.text = f
	)
	
	on bnGo pressed do (
		if actArchive.checked AND zipFilename.text == "" then (
			messagebox "No archive file is defined." title:"Delete Minor Versions"
			return undefined
		)
		if (queryBox "Delete minor versions (no undo!)?" title:"Delete Minor Versions") then (
			doDelete()
			updateLists()
		)
	)
	
	on bnClose pressed do (destroyDialog delMinorVersions)
) -- end delMinorVersions rollout

createDialog delMinorVersions 450 480